@@ -27,12 +27,7 @@
 #include "macros.h"
 
 #undef signals
-#include <nm-version.h>
-#if NM_CHECK_VERSION(1, 0, 0)
 #include <libnm/NetworkManager.h>
-#else
-#include <NetworkManager.h>
-#endif
 #define signals Q_SIGNALS
 
 #include "activeconnection.h"
@@ -41,6 +36,7 @@
 #include "bonddevice.h"
 #include "bridgedevice.h"
 #include "infinibanddevice.h"
+#include "iptunneldevice.h"
 #include "genericdevice.h"
 #include "modemdevice.h"
 #include "olpcmeshdevice.h"
@@ -58,27 +54,31 @@
 
 #include "nmdebug.h"
 
+#define DBUS_OBJECT_MANAGER  "org.freedesktop.DBus.ObjectManager"
 #define DBUS_PROPERTIES  "org.freedesktop.DBus.Properties"
 
 #ifdef NMQT_STATIC
 const QString NetworkManager::NetworkManagerPrivate::DBUS_SERVICE(QString::fromLatin1("org.kde.fakenetwork"));
 const QString NetworkManager::NetworkManagerPrivate::DBUS_DAEMON_PATH(QString::fromLatin1("/org/kde/fakenetwork"));
+const QString NetworkManager::NetworkManagerPrivate::DBUS_DAEMON_INTERFACE(QString::fromLatin1("org.kde.fakenetwork"));
 const QString NetworkManager::NetworkManagerPrivate::DBUS_SETTINGS_PATH(QString::fromLatin1("/org/kde/fakenetwork/Settings"));
 #else
 const QString NetworkManager::NetworkManagerPrivate::DBUS_SERVICE(QString::fromLatin1(NM_DBUS_SERVICE));
 const QString NetworkManager::NetworkManagerPrivate::DBUS_DAEMON_PATH(QString::fromLatin1(NM_DBUS_PATH));
+const QString NetworkManager::NetworkManagerPrivate::DBUS_DAEMON_INTERFACE(QString::fromLatin1(NM_DBUS_INTERFACE));
 const QString NetworkManager::NetworkManagerPrivate::DBUS_SETTINGS_PATH(QString::fromLatin1(NM_DBUS_PATH_SETTINGS));
 #endif
 const QString NetworkManager::NetworkManagerPrivate::FDO_DBUS_PROPERTIES(QString::fromLatin1(DBUS_PROPERTIES));
+const QString NetworkManager::NetworkManagerPrivate::FDO_DBUS_OBJECT_MANAGER(QString::fromLatin1(DBUS_OBJECT_MANAGER));
 
 Q_GLOBAL_STATIC(NetworkManager::NetworkManagerPrivate, globalNetworkManager)
 
 NetworkManager::NetworkManagerPrivate::NetworkManagerPrivate()
 #ifdef NMQT_STATIC
-    : watcher(DBUS_SERVICE, QDBusConnection::sessionBus(), QDBusServiceWatcher::WatchForOwnerChange, this)
+    : watcher(DBUS_SERVICE, QDBusConnection::sessionBus(), QDBusServiceWatcher::WatchForUnregistration, this)
     , iface(NetworkManager::NetworkManagerPrivate::DBUS_SERVICE, NetworkManager::NetworkManagerPrivate::DBUS_DAEMON_PATH, QDBusConnection::sessionBus())
 #else
-    : watcher(DBUS_SERVICE, QDBusConnection::systemBus(), QDBusServiceWatcher::WatchForOwnerChange, this)
+    : watcher(DBUS_SERVICE, QDBusConnection::systemBus(), QDBusServiceWatcher::WatchForUnregistration, this)
     , iface(NetworkManager::NetworkManagerPrivate::DBUS_SERVICE, NetworkManager::NetworkManagerPrivate::DBUS_DAEMON_PATH, QDBusConnection::systemBus())
 #endif
     , nmState(NetworkManager::Unknown)
@@ -90,6 +90,7 @@ NetworkManager::NetworkManagerPrivate::NetworkManagerPrivate()
     , m_isWirelessHardwareEnabled(false)
     , m_isWwanEnabled(false)
     , m_isWwanHardwareEnabled(false)
+    , m_globalDnsConfiguration(NetworkManager::DnsConfiguration())
     , m_supportedInterfaceTypes(NetworkManager::Device::UnknownType)
 {
     connect(&iface, &OrgFreedesktopNetworkManagerInterface::DeviceAdded,
@@ -99,22 +100,16 @@ NetworkManager::NetworkManagerPrivate::NetworkManagerPrivate()
 
 
 #ifndef NMQT_STATIC
-#if NM_CHECK_VERSION(1, 4, 0)
     QDBusConnection::systemBus().connect(NetworkManagerPrivate::DBUS_SERVICE, NetworkManagerPrivate::DBUS_DAEMON_PATH, NetworkManagerPrivate::FDO_DBUS_PROPERTIES,
                                          QLatin1String("PropertiesChanged"), this, SLOT(dbusPropertiesChanged(QString,QVariantMap,QStringList)));
 #else
     connect(&iface, &OrgFreedesktopNetworkManagerInterface::PropertiesChanged,
             this, &NetworkManagerPrivate::propertiesChanged);
 #endif
-#endif
 
-#ifdef NMQT_STATIC
-    connect(&iface, &OrgFreedesktopNetworkManagerInterface::PropertiesChanged,
-            this, &NetworkManagerPrivate::propertiesChanged);
-#endif
+    iface.connection().connect(NetworkManagerPrivate::DBUS_SERVICE, "/org/freedesktop", NetworkManagerPrivate::FDO_DBUS_OBJECT_MANAGER,
+                               QLatin1String("InterfacesAdded"), this, SLOT(dbusInterfacesAdded(QDBusObjectPath,QVariantMap)));
 
-    connect(&watcher, &QDBusServiceWatcher::serviceRegistered,
-            this, &NetworkManagerPrivate::daemonRegistered);
     connect(&watcher, &QDBusServiceWatcher::serviceUnregistered,
             this, &NetworkManagerPrivate::daemonUnregistered);
 
@@ -166,7 +161,11 @@ void NetworkManager::NetworkManagerPrivate::init()
                NetworkManager::Device::Adsl |
                NetworkManager::Device::Bridge |
                NetworkManager::Device::Generic |
-               NetworkManager::Device::Team
+               NetworkManager::Device::Team |
+               NetworkManager::Device::MacVlan |
+               NetworkManager::Device::Tun |
+               NetworkManager::Device::Veth |
+               NetworkManager::Device::IpTunnel
            );
 
     // Get all Manager's properties async
@@ -175,7 +174,7 @@ void NetworkManager::NetworkManagerPrivate::init()
         propertiesChanged(initialProperties);
     }
 
-    QTimer::singleShot(0, [this] { qobject_cast<SettingsPrivate *>(settingsNotifier())->init(); });
+    QTimer::singleShot(0, [] { qobject_cast<SettingsPrivate *>(settingsNotifier())->init(); });
 
     if (iface.isValid()) {
         QList <QDBusObjectPath> devices = iface.devices();
@@ -202,7 +201,7 @@ int NetworkManager::NetworkManagerPrivate::compareVersion(const QString &version
 {
     int x, y, z;
 
-    QStringList sl = version.split('.');
+    const auto sl = version.splitRef('.');
 
     if (sl.size() > 2) {
         x = sl[0].toInt();
@@ -272,9 +271,10 @@ QVariantMap NetworkManager::NetworkManagerPrivate::retrieveInitialProperties(con
 NetworkManager::Device::Ptr NetworkManager::NetworkManagerPrivate::findRegisteredNetworkInterface(const QString &uni)
 {
     NetworkManager::Device::Ptr networkInterface;
-    if (networkInterfaceMap.contains(uni)) {
-        if (networkInterfaceMap.value(uni)) {
-            networkInterface = networkInterfaceMap.value(uni);
+    auto it = networkInterfaceMap.constFind(uni);
+    if (it != networkInterfaceMap.constEnd()) {
+        if (*it) {
+            networkInterface = *it;
         } else {
             networkInterface = createNetworkInterface(uni);
             networkInterfaceMap[uni] = networkInterface;
@@ -287,9 +287,10 @@ NetworkManager::ActiveConnection::Ptr NetworkManager::NetworkManagerPrivate::fin
 {
     NetworkManager::ActiveConnection::Ptr activeConnection;
     if (!uni.isEmpty() && uni != QLatin1String("/")) {
-        const bool contains = m_activeConnections.contains(uni);
-        if (contains && m_activeConnections.value(uni)) {
-            activeConnection = m_activeConnections.value(uni);
+        const auto it = m_activeConnections.constFind(uni);
+        const bool contains = it != m_activeConnections.constEnd();
+        if (contains && *it) {
+            activeConnection = *it;
         } else {
             activeConnection = NetworkManager::ActiveConnection::Ptr(new NetworkManager::VpnConnection(uni), &QObject::deleteLater);
             if (activeConnection->connection()) {
@@ -360,6 +361,9 @@ NetworkManager::Device::Ptr NetworkManager::NetworkManagerPrivate::createNetwork
     case Device::Veth:
         createdInterface = Device::Ptr(new NetworkManager::VethDevice(uni), &QObject::deleteLater);
         break;
+    case Device::IpTunnel:
+        createdInterface = Device::Ptr(new NetworkManager::IpTunnelDevice(uni), &QObject::deleteLater);
+        break;
     default:
         createdInterface = device;
         if (uni != QLatin1String("any")) { // VPN connections use "any" as uni for the network interface.
@@ -677,6 +681,17 @@ NetworkManager::Device::MeteredStatus NetworkManager::NetworkManagerPrivate::met
     return checkVersion(1, 0, 6) ? m_metered : NetworkManager::Device::UnknownStatus;
 }
 
+NetworkManager::DnsConfiguration NetworkManager::NetworkManagerPrivate::globalDnsConfiguration() const
+{
+    return m_globalDnsConfiguration;
+}
+
+void NetworkManager::NetworkManagerPrivate::setGlobalDnsConfiguration(const NetworkManager::DnsConfiguration &configuration)
+{
+    m_globalDnsConfiguration = configuration;
+    iface.setGlobalDnsConfiguration(m_globalDnsConfiguration.toMap());
+}
+
 void NetworkManager::NetworkManagerPrivate::onDeviceAdded(const QDBusObjectPath &objpath)
 {
     // qCDebug(NMQT);
@@ -801,8 +816,11 @@ void NetworkManager::NetworkManagerPrivate::propertiesChanged(const QVariantMap
         } else if (property == QLatin1String("Metered")) {
             m_metered = (NetworkManager::Device::MeteredStatus)it->toUInt();
             Q_EMIT meteredChanged(m_metered);
+        } else if (property == QLatin1String("GlobalDnsConfiguration")) {
+            m_globalDnsConfiguration.fromMap(qdbus_cast<QVariantMap>(*it));
+            Q_EMIT globalDnsConfigurationChanged(m_globalDnsConfiguration);
         } else {
-            qCWarning(NMQT) << Q_FUNC_INFO << "Unhandled property" << property;
+            qCDebug(NMQT) << Q_FUNC_INFO << "Unhandled property" << property;
         }
         ++it;
     }
@@ -863,8 +881,13 @@ NetworkManager::Status NetworkManager::NetworkManagerPrivate::convertNMState(uin
     return status;
 }
 
-void NetworkManager::NetworkManagerPrivate::daemonRegistered()
+void NetworkManager::NetworkManagerPrivate::interfacesAdded(const QDBusObjectPath &path, const QVariantMap &addedInterfaces)
 {
+    Q_UNUSED(path);
+
+    if(!addedInterfaces.contains(NetworkManagerPrivate::DBUS_DAEMON_INTERFACE))
+        return;
+
     init();
     Q_EMIT serviceAppeared();
 }
@@ -1103,6 +1126,16 @@ NetworkManager::Device::MeteredStatus NetworkManager::metered()
     return globalNetworkManager->metered();
 }
 
+NetworkManager::DnsConfiguration NetworkManager::globalDnsConfiguration()
+{
+    return globalNetworkManager->globalDnsConfiguration();
+}
+
+void NetworkManager::setGlobalDnsConfiguration(const NetworkManager::DnsConfiguration &configuration)
+{
+    globalNetworkManager->setGlobalDnsConfiguration(configuration);
+}
+
 NetworkManager::Notifier *NetworkManager::notifier()
 {
     return globalNetworkManager;
