@@ -21,11 +21,7 @@
 #include "bluetoothsetting.h"
 #include "bluetoothsetting_p.h"
 
-#if !NM_CHECK_VERSION(1, 0, 0)
-#include <nm-setting-bluetooth.h>
-#endif
-
-#include <QtCore/QDebug>
+#include <QDebug>
 
 NetworkManager::BluetoothSettingPrivate::BluetoothSettingPrivate()
     : name(NM_SETTING_BLUETOOTH_SETTING_NAME)
